#!   /usr/bin/env   python3
#    coding: utf8

# Copyright CERN, 2011
# Author: Matthieu Cattin <matthieu.cattin@cern.ch>
# Licence: GPL v2 or later.
# Website: http://www.ohwr.org
# Last modifications: 21/07/2022

# Import system modules
import datetime
import argparse

from fmc_eeprom import *


def config_parser(parser):
    parser.description = "Generate ECA-03287 EEPROM content."
    parser.add_argument("serial", nargs="?", default="HCCFFIH01F-CR000001", help="Infor EAM serial number")
    parser.add_argument("partno", nargs="?", default="EDA-03287-V5", help="EDA part number")
    parser.add_argument("mfg_date", nargs="?", default="18/5/2022", help="Manufacturing date without time")
    parser.add_argument("--output", nargs="?", default="eeprom.out", help="EEPROM content (sysfs) file")
    parser.set_defaults(handler=main)

def parse_cmd():
    parser = argparse.ArgumentParser()
    config_parser(parser)
    args = parser.parse_args()
    args.handler(args)

def main(main_args):
    # Constants
    PRODUCT = "FmcDio10i8o"
    MEMLEN = 216 # hardcoded cause libipmi has no get_size() or similar

    #==================================================
    # Calculate number of minutes since 0:00 1/1/96
    mfg_date = datetime.datetime.strptime(main_args.mfg_date, "%d/%m/%Y")
    print(f"Manufacturing date/time: {mfg_date}")
    ref_date = datetime.datetime(1996, 1, 1)
    diff_date = mfg_date - ref_date
    total_seconds = diff_date.days * 86400 + diff_date.seconds
    mfg_date_min = int(total_seconds//60)
    print(f"Manufacturing date in minutes: {mfg_date_min} 0x{mfg_date_min:06x}")

    #==================================================
    # Create Board Info Area
    # FRU field is used to store the date of generation of the eeprom content
    # This could be used later to determine if the content has to be udated (bug fix, ...)
    now_date = datetime.datetime.now()
    print(f"EEPROM content generated: {now_date}")
    fru = str(now_date)
    product = PRODUCT
    print(f"Board's product name: {product}")
    serial = main_args.serial
    print(f"Board's serial number: {serial}")
    partno = main_args.partno
    print(f"Board's part number: {partno}")
    bia = BoardInfoArea(mfg_date_min, "CERN", product, serial, partno, fru)

    #==================================================
    # Multirecords Area

    # output number, vnom, vmin, vmax, ripple, imin, imax
    dcload0 = DCLoadRecord(0, 2.5, 1.8, 2.5, 0.010, 0, 800) # VADJ
    dcload1 = DCLoadRecord(1, 3.3, 3.135, 3.465, 0.010, 0, 3000) # P3V3
    dcload2 = DCLoadRecord(2, 12.0, 11.4, 12.6, 0.010, 0, 1000)  # P12V
    dcload = [ dcload0, dcload1, dcload2 ]

    # output number, vnom, vmin, vmax, ripple, imin, imax
    dcout0 = DCOutputRecord(3, 0.0, 0.0, 0.0, 0.0, 0, 0) # VIO_B_M2C
    dcout1 = DCOutputRecord(4, 0.0, 0.0, 0.0, 0.0, 0, 0) # VREF_A_M2C
    dcout2 = DCOutputRecord(5, 0.0, 0.0, 0.0, 0.0, 0, 0) # VREF_B_M2C
    dcout = [ dcout0, dcout1, dcout2 ]

    # OEM Record Subtype 0
    # module size  : 0=single width, 1=double width
    # P1 size      : 0=LPC, 1=HPC
    # P2 size      : 0=LPC, 1=HPC, 3=not fitted
    # clock dir    : 0=M2C, 1=C2M
    # nb sig P1 A  : number
    # nb sig P1 B  : number
    # nb sig P2 A  : number
    # nb sig P2 B  : number
    # nb GBT P1    : number
    # nb GBT P2    : number
    # max TCK freq : frequency in MHz
    oem = OEMRecord(0, 0, 3, 1, 68, 0, 0, 0, 0, 0, 0)

    # OEM Record Subtype 1 (I2C Device Defenition), optional - not used

    #==================================================
    # Internal Use Area
    # Takes an array of byte as parameter
    iua_data = [0x1,0x2,0x3,0x4]
    iua = InternalUseArea(iua_data)

    #==================================================
    # Write eeprom content to a binary file
    eeprom_fn = main_args.output
    print(f"Writing to {eeprom_fn}..")
    ipmi_open_file(eeprom_fn)
    #ipmi_set(bia, dcload, dcout, oem, iua)
    ipmi_set(bia, dcload, dcout, oem)
    ipmi_write()
    ipmi_close_file()
    print("Done!\n")

    #==================================================
    # Read eeprom content from binary file
    f_eeprom = open(eeprom_fn, "rb")
    eeprom_data = []
    byte = f_eeprom.read(1) # reads one byte

    for i in range(MEMLEN):
        eeprom_data.append(ord(byte))
        byte = f_eeprom.read(1) # reads one byte
    f_eeprom.close()

    print("Raw EEPROM data:")
    i = 0
    for data in eeprom_data:
        if i%8 == 0:
            print("0x%02X (%3d) : %02X" % (i, i, data), end=' ')
        else:
            print("%02X" % (data), end=' ')
        i += 1
        if i%8 == 0:
            print("")

    print("\nTODO: check data validity - use hd for now")

if __name__ == '__main__' :
    parse_cmd()
